/**
 * EasyBeans
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ENCBindingBuilder.java 3524 2008-06-20 13:37:06Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl.enc;

import java.util.Collection;

import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonClassMetadata;
import org.ow2.util.ee.metadata.common.impl.enc.AbsENCBindingBuilder;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBindingException;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBindingHolder;

/**
 * Utility class that helps to build the ENC Binding by analyzing metadata.
 * @author Florent BENOIT
 */
public class CarENCBindingBuilder extends AbsENCBindingBuilder {

    /**
     * All class metadata are kept by this object.
     */
    private ICarDeployableMetadata carDeployableMetadata = null;


    /**
     * Constructor used only by this class/subclasses.
     * @param carDeployableMetadata the given metadata to analyze.
     */
    protected CarENCBindingBuilder(final ICarDeployableMetadata carDeployableMetadata) {
        super();
        this.carDeployableMetadata = carDeployableMetadata;
    }

    /**
     * @return list of metadata to analyze.
     */
    @Override
    protected Collection<? extends ICommonClassMetadata<?, ?, ?>> getClassMetadataCollection() {
        return carDeployableMetadata.getCarClassMetadataCollection();
    }


    /**
     * Found all ENC bindings from the given set of metadata.
     * @param carDeployableMetadata the given metadata to analyze.
     * @return object that contains all bindings found for the given set of
     *         classes.
     * @throws ENCBindingException if the analyze of the set of the metadata has
     *         failed.
     */
    public static ENCBindingHolder analyze(final ICarDeployableMetadata carDeployableMetadata) throws ENCBindingException {
        CarENCBindingBuilder bindingBuilder = new CarENCBindingBuilder(carDeployableMetadata);
        bindingBuilder.analyze();
        return bindingBuilder.getHolder();
    }

}
