/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: CarMetadataMerge.java 4028 2008-09-30 10:02:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl.xml.merge;

import org.ow2.util.ee.metadata.car.api.ICarClassMetadata;
import org.ow2.util.ee.metadata.car.impl.CarDeployableMetadata;
import org.ow2.util.ee.metadata.car.impl.xml.struct.CAR;
import org.ow2.util.ee.metadata.common.api.interfaces.IEnvironmentRefAccessor;
import org.ow2.util.ee.metadata.common.impl.xml.merge.MetadataMerge;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.archive.api.IArchiveMetadata;

import java.util.jar.Manifest;
import java.util.jar.Attributes;

/**
 * CarMetadataMerge.
 * @author Gael Lalire
 */
public class CarMetadataMerge extends MetadataMerge {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarMetadataMerge.class);

    /**
     * The metadata where merge will be done.
     */
    private CarDeployableMetadata carDeployableMetadata;

    /**
     * Constructor.
     * @param carDeployableMetadata the metadata where merge will be done
     */
    public CarMetadataMerge(final CarDeployableMetadata carDeployableMetadata) {
        this.carDeployableMetadata = carDeployableMetadata;
    }

    /**
     * Merge current metadata with car.
     * @param car object representing the xml
     */
    public void merge(final CAR car) {
        applyJndiEnvironmentRefsGroup(car, carDeployableMetadata);

        // only in CAR
        String callbackHandler = car.getCallbackHandler();
        if (callbackHandler != null) {
            ICarClassMetadata carClassMetadata = getClassMetadata(callbackHandler.replace('.', '/'));
            if (carClassMetadata == null) {
                LOGGER.error("The callback handler class '" + callbackHandler + "' defined in xml was not found.");
            } else {
                carClassMetadata.setCallbackHandler(true);
            }
        }
    }

    /**
     * @param className the class name
     * @return class metadata of class name
     */
    @Override
    public ICarClassMetadata getClassMetadata(final String className) {
        return carDeployableMetadata.getCarClassMetadata(className);
    }

    /**
     * Provide the main-class of the client.
     * @param env the car environment
     * @return the main-class of the client archive
     */
    @Override
    public String findDefaultClassForEnvironment(final IEnvironmentRefAccessor env) {
        IArchiveMetadata metadata = carDeployableMetadata.getDeployable().getArchive().getMetadata();
        String mainClassName = metadata.get(Attributes.Name.MAIN_CLASS.toString());
        if (mainClassName == null) {
            // try with the attribute name in lowercase (not spec compliant...)
            mainClassName = metadata.get("main-class");
            if (mainClassName != null) {
                LOGGER.warn("The '" + Attributes.Name.MAIN_CLASS.toString() + "' attribute is not written correctly in the archive '" + carDeployableMetadata.getDeployable().getArchive() + "'");
            }

        }
        if (mainClassName == null) {
            throw new IllegalStateException("No '" + Attributes.Name.MAIN_CLASS.toString() + "' attribute found in the archive '" + carDeployableMetadata.getDeployable().getArchive() + "'");
        }

        return encode(mainClassName);
    }

}
