/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CARDeploymentDescLoader.java 3788 2008-07-30 16:23:46Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl.xml.parsing;

import java.net.URL;

import org.ow2.util.ee.metadata.car.impl.xml.struct.CAR;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.CommonDeploymentDescLoader;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class analyzes the CAR deployment descriptor and fill a struct with
 * getters.
 * @author Gael Lalire
 */
public final class CARDeploymentDescLoader extends CommonDeploymentDescLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(CARDeploymentDescLoader.class);

    /**
     * EJB3 instance object that is the result of the parsing.
     */
    private CAR car = null;

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the EJB Xml file.
     */
    private CARDeploymentDescLoader(final URL url) {
        super(url);
        this.car = new CAR();
    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static CAR loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        CARDeploymentDescLoader carDeploymentDescLoader = new CARDeploymentDescLoader(url);
        carDeploymentDescLoader.parse();
        return carDeploymentDescLoader.getCAR();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), isValidating(), new CAREntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <application-client>
        Element applicationClientRootElement = document.getDocumentElement();

        if (applicationClientRootElement.getNamespaceURI() != null) {
            // Define NS
            defineXMLNS(applicationClientRootElement);
        }

        analyzeEnvironment(applicationClientRootElement, car);

        car.setCallbackHandler(XMLUtils.getStringValueElement(getXMLNS(), applicationClientRootElement, "callback-handler"));

        if ("true".equals(applicationClientRootElement.getAttribute("metadata-complete"))) {
            car.setMetadataComplete(true);
        } else {
            car.setMetadataComplete(false);
        }

    }

    /**
     * @return the object resulting of the XML parsing.
     */
    public CAR getCAR() {
        return car;
    }

}
